import asyncio
from typing import Optional

from fastapi import APIRouter, HTTPException
from pam import PamAuthenticator
from pydantic import BaseModel

from common_util import cmd_line_args, generic_ok_response
from domain_socket_listener import (
    MsgFromGlintServer,
    create_subscriber,
    delete_subscriber,
    get_socket_msg,
)
from glint_session import glint_session_handler
from settings import settings

auth_router = APIRouter()
# TODO: необходимо чтобы глинт лаунчер отвечал реквестеру на языке, который он хочет видеть.
#  Т е нужно прикрутить систему переводов.


class AuthData(BaseModel):
    username: str
    password: str
    width: Optional[int] = 1280
    height: Optional[int] = 1024
    depth: Optional[int] = 24
    desktop: Optional[str] = None


async def wait_for_server_ready_msg(msg_queue):
    while settings.is_running:
        msg = await get_socket_msg(msg_queue)
        if msg == MsgFromGlintServer.SERVER_START_FAIL.value:
            # Failed to start Glint Server
            raise HTTPException(status_code=400, detail="Не удалось запустить Glint сервер")
        elif msg == MsgFromGlintServer.SERVER_READY.value:
            print("!!! msg == MsgFromGlintServer.SERVER_READY.value:")  # noqa: T201
            return


@auth_router.post("/auth")
async def auth(auth_data: AuthData):
    # Извлечь логин пароль. Проверить их.
    authorized = PamAuthenticator().authenticate(auth_data.username, auth_data.password)
    if not authorized:
        # Wrong credentials
        raise HTTPException(status_code=401, detail="Неверные учетные данные")

    glint_session_handler.init_session(auth_data.username)

    # Запуск графической сессии
    print("Beg auth")  # noqa: T201

    if not glint_session_handler.start_xvfb(auth_data.username, auth_data.width, auth_data.height, auth_data.depth):
        raise HTTPException(status_code=500, detail="Не удалось запустить виртуальный графический сервер")

    desktop = auth_data.desktop if auth_data.desktop else cmd_line_args.desktop
    glint_session_handler.start_desktop(auth_data.username, desktop)

    domain_socket_msg_queue = create_subscriber()

    glint_session_handler.start_glint(auth_data.username)
    glint_session_handler.start_pulseaudio(auth_data.username)

    # Wait for message from glint server
    try:
        await asyncio.wait_for(wait_for_server_ready_msg(domain_socket_msg_queue), 10)
    except asyncio.TimeoutError:
        pass

    delete_subscriber(domain_socket_msg_queue)
    print("End auth")  # noqa: T201
    return generic_ok_response
